import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import cn from 'classnames';
import shallowEqual from 'fbjs/lib/shallowEqual';
import * as React from 'react';
import { accessibilityOverscanIndicesGetter, Grid } from "react-virtualized/dist/es/Grid";
import defaultRowRenderer from './defaultRowRenderer';
import { defaultControlStyle, defaultRowStyle, NodeRecord, Update } from './utils';

/**
 * This component renders a tree of elements using powerful virtualization technology to render
 * only visible elements to the HTML document.
 */
var Tree =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Tree, _React$Component);

  function Tree() {
    var _temp, _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return (_temp = _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this, _this.state = {
      order: [],
      useDynamicRowHeight: false
    }, _this.grid = void 0, _this.registry = {}, _this.getRowHeight = function (_ref) {
      var index = _ref.index;
      var rowHeight = _this.props.rowHeight;
      var order = _this.state.order;
      var id = order[index];
      var node = _this.registry[id].node;
      return node.height || rowHeight;
    }, _this.cellRenderer = function (_ref2) {
      var rowIndex = _ref2.rowIndex,
          style = _ref2.style,
          isScrolling = _ref2.isScrolling,
          key = _ref2.key;
      var _this$props = _this.props,
          controlClassName = _this$props.controlClassName,
          controlStyle = _this$props.controlStyle,
          rowClassName = _this$props.rowClassName,
          nodeNestingMultiplier = _this$props.nodeNestingMultiplier,
          onRowClick = _this$props.onRowClick,
          onRowDoubleClick = _this$props.onRowDoubleClick,
          onRowMouseOver = _this$props.onRowMouseOver,
          onRowMouseOut = _this$props.onRowMouseOut,
          onRowRightClick = _this$props.onRowRightClick,
          rowHeight = _this$props.rowHeight,
          rowRenderer = _this$props.rowRenderer,
          rowStyle = _this$props.rowStyle;
      var order = _this.state.order; // TRICKY The style object is sometimes cached by Grid.
      // This prevents new style objects from bypassing shallowCompare().
      // However as of React 16, style props are auto-frozen (at least in dev mode)
      // Check to make sure we can still modify the style before proceeding.
      // https://github.com/facebook/react/commit/977357765b44af8ff0cfea327866861073095c12#commitcomment-20648713

      var _ref3 = Object.getOwnPropertyDescriptor(style, 'width'),
          writable = _ref3.writable;

      if (writable) {
        // By default, Tree cells should be 100% width.
        // This prevents them from flowing under a scrollbar (if present).
        style.width = '100%';
      }

      var id = order[rowIndex];
      var record = _this.registry[id];
      var _record$node = record.node,
          childrenCount = _record$node.childrenCount,
          nestingLevel = _record$node.nestingLevel,
          nodeData = _record$node.nodeData,
          height = _record$node.height,
          specificRowStyle = _record$node.style,
          isOpened = record.isOpened,
          onNodeToggle = record.onNodeToggle;
      return rowRenderer({
        childrenCount: childrenCount,
        className: rowClassName,
        controlClassName: controlClassName,
        controlStyle: Object.assign({}, defaultControlStyle, controlStyle),
        id: id,
        index: rowIndex,
        isOpened: isOpened,
        isScrolling: isScrolling,
        key: key,
        nestingLevel: nestingLevel,
        nodeData: nodeData,
        onNodeToggle: onNodeToggle,
        onRowClick: onRowClick,
        onRowDoubleClick: onRowDoubleClick,
        onRowMouseOut: onRowMouseOut,
        onRowMouseOver: onRowMouseOver,
        onRowRightClick: onRowRightClick,
        style: Object.assign({}, style, defaultRowStyle, rowStyle, specificRowStyle, {
          height: height || rowHeight,
          marginLeft: nestingLevel * nodeNestingMultiplier,
          overflow: 'hidden'
        })
      });
    }, _this.finishNodeToggling =
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee() {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return _this.recomputeTree(Update.Nodes);

            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    })), _this.onScroll = function (_ref5) {
      var clientHeight = _ref5.clientHeight,
          scrollHeight = _ref5.scrollHeight,
          scrollTop = _ref5.scrollTop;
      var onScroll = _this.props.onScroll;
      onScroll({
        clientHeight: clientHeight,
        scrollHeight: scrollHeight,
        scrollTop: scrollTop
      });
    }, _this.onSectionRendered = function (_ref6) {
      var rowOverscanStartIndex = _ref6.rowOverscanStartIndex,
          rowOverscanStopIndex = _ref6.rowOverscanStopIndex,
          rowStartIndex = _ref6.rowStartIndex,
          rowStopIndex = _ref6.rowStopIndex;
      var onRowsRendered = _this.props.onRowsRendered;
      onRowsRendered({
        overscanStartIndex: rowOverscanStartIndex,
        overscanStopIndex: rowOverscanStopIndex,
        startIndex: rowStartIndex,
        stopIndex: rowStopIndex
      });
    }, _this.setRef = function (grid) {
      _this.grid = grid;
    }, _temp) || _assertThisInitialized(_this);
  }

  var _proto = Tree.prototype;

  _proto.componentWillMount =
  /*#__PURE__*/
  function () {
    var _componentWillMount = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee2() {
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.recomputeTree(Update.NodesAndOpenness);

            case 2:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function componentWillMount() {
      return _componentWillMount.apply(this, arguments);
    };
  }();

  _proto.componentWillReceiveProps =
  /*#__PURE__*/
  function () {
    var _componentWillReceiveProps = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee3(_ref7) {
      var update;
      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              update = _ref7.update;

              if (!(update !== Update.None)) {
                _context3.next = 4;
                break;
              }

              _context3.next = 4;
              return this.recomputeTree(update);

            case 4:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    return function componentWillReceiveProps(_x) {
      return _componentWillReceiveProps.apply(this, arguments);
    };
  }();

  _proto.forceUpdateGrid = function forceUpdateGrid(callback) {
    if (this.grid) {
      this.grid.forceUpdate(callback);
    }
  };
  /** See Grid#getOffsetForCell */


  _proto.getOffsetForRow = function getOffsetForRow(_ref8) {
    var alignment = _ref8.alignment,
        index = _ref8.index;

    if (this.grid) {
      var _grid$getOffsetForCel = this.grid.getOffsetForCell({
        alignment: alignment,
        columnIndex: 0,
        rowIndex: index
      }),
          _scrollTop = _grid$getOffsetForCel.scrollTop;

      return _scrollTop;
    }

    return 0;
  };
  /** CellMeasurer compatibility */


  _proto.invalidateCellSizeAfterRender = function invalidateCellSizeAfterRender(_ref9) {
    var columnIndex = _ref9.columnIndex,
        rowIndex = _ref9.rowIndex;

    if (this.grid) {
      this.grid.invalidateCellSizeAfterRender({
        columnIndex: columnIndex,
        rowIndex: rowIndex
      });
    }
  };
  /** See Grid#measureAllCells */


  _proto.measureAllRows = function measureAllRows() {
    if (this.grid) {
      this.grid.measureAllCells();
    }
  };
  /** CellMeasurer compatibility */


  _proto.recomputeGridSize = function recomputeGridSize(_temp2) {
    var _ref10 = _temp2 === void 0 ? {} : _temp2,
        _ref10$columnIndex = _ref10.columnIndex,
        columnIndex = _ref10$columnIndex === void 0 ? 0 : _ref10$columnIndex,
        _ref10$rowIndex = _ref10.rowIndex,
        rowIndex = _ref10$rowIndex === void 0 ? 0 : _ref10$rowIndex;

    if (this.grid) {
      this.grid.recomputeGridSize({
        columnIndex: columnIndex,
        rowIndex: rowIndex
      });
    }
  };
  /**
   * Recomputes Tree component basing on update type.
   *
   * Method calls "nodeGetter" internally and flattens tree structure to an array. Depending on "update" value it runs
   * one of the following algorithms:
   * 1) Update.NodesAndOpenness. Requires full node metadata. Updates order, number and rendering data of all
   * nodes. Overrides current openness state with "openedByDefault" value.
   * 2) Update.Nodes. Requires full node metadata. Updates order, number and rendering data of all nodes. Preserves
   * openness state of all nodes.
   * 3) Update.Order. Requires only node id. Updates nodes order (useful for sorting etc.). Preserves openness
   * state of all nodes.
   * 4) Update.None. Updates nothing.
   *
   * @param update
   *
   * @returns promise resolves when re-rendering is ended
   */


  _proto.recomputeTree =
  /*#__PURE__*/
  function () {
    var _recomputeTree = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee4(update) {
      var _this2 = this;

      var _props, nodeGetter, rowHeight, order, useDynamicRowHeight, refresh, ignoreInnerState, g, isPreviousOpened, _g$next, _value, _done, _height, _id, isOpenedByDefault, record;

      return _regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (!(update === Update.None)) {
                _context4.next = 2;
                break;
              }

              return _context4.abrupt("return", void 0);

            case 2:
              _props = this.props, nodeGetter = _props.nodeGetter, rowHeight = _props.rowHeight;
              order = [];
              useDynamicRowHeight = false;
              refresh = update === Update.Nodes || update === Update.NodesAndOpenness;
              ignoreInnerState = update === Update.NodesAndOpenness;
              g = nodeGetter(refresh);
              isPreviousOpened = false; // tslint:disable-next-line:no-constant-condition

            case 9:
              if (!true) {
                _context4.next = 16;
                break;
              }

              _g$next = g.next(isPreviousOpened), _value = _g$next.value, _done = _g$next.done;

              if (!(_done || !_value)) {
                _context4.next = 13;
                break;
              }

              return _context4.abrupt("break", 16);

            case 13:
              if (typeof _value === 'string') {
                order.push(_value);
                isPreviousOpened = this.registry[_value].isOpened;
              } else {
                _height = _value.height, _id = _value.id, isOpenedByDefault = _value.isOpenedByDefault;
                record = this.registry[_id];

                if (!record) {
                  this.registry[_id] = new NodeRecord(_value, isOpenedByDefault, this.finishNodeToggling);
                } else {
                  record.node = _value;

                  if (ignoreInnerState) {
                    record.isOpened = isOpenedByDefault;
                  }
                }

                order.push(_id);
                isPreviousOpened = this.registry[_id].isOpened;

                if (_height && _height !== rowHeight) {
                  useDynamicRowHeight = true;
                }
              }

              _context4.next = 9;
              break;

            case 16:
              return _context4.abrupt("return", new Promise(function (resolve) {
                _this2.setState({
                  order: order,
                  useDynamicRowHeight: useDynamicRowHeight
                }, function () {
                  if (useDynamicRowHeight) {
                    _this2.recomputeGridSize();

                    resolve();
                  } else {
                    _this2.forceUpdateGrid(resolve);
                  }
                });
              }));

            case 17:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    return function recomputeTree(_x2) {
      return _recomputeTree.apply(this, arguments);
    };
  }();

  _proto.render = function render() {
    var _props2 = this.props,
        className = _props2.className,
        noRowsRenderer = _props2.noRowsRenderer,
        onScroll = _props2.onScroll,
        rowHeight = _props2.rowHeight,
        scrollToIndex = _props2.scrollToIndex,
        update = _props2.update,
        width = _props2.width,
        other = _objectWithoutProperties(_props2, ["className", "noRowsRenderer", "onScroll", "rowHeight", "scrollToIndex", "update", "width"]);

    var _state = this.state,
        order = _state.order,
        useDynamicRowHeight = _state.useDynamicRowHeight;
    var classNames = cn('ReactVirtualized__Tree', className);
    return React.createElement(Grid, _extends({
      autoContainerWidth: true,
      cellRenderer: this.cellRenderer,
      className: classNames,
      columnWidth: width,
      columnCount: 1,
      noContentRenderer: noRowsRenderer,
      onScroll: this.onScroll,
      onSectionRendered: this.onSectionRendered,
      ref: this.setRef,
      rowCount: order.length,
      rowHeight: useDynamicRowHeight ? this.getRowHeight : rowHeight,
      scrollToRow: scrollToIndex,
      width: width
    }, other));
  };
  /** See Grid#scrollToPosition */


  _proto.scrollToPosition = function scrollToPosition(scrollTop) {
    if (scrollTop === void 0) {
      scrollTop = 0;
    }

    if (this.grid) {
      this.grid.scrollToPosition({
        scrollTop: scrollTop
      });
    }
  };
  /** See Grid#scrollToCell */


  _proto.scrollToRow = function scrollToRow(index) {
    if (index === void 0) {
      index = 0;
    }

    if (this.grid) {
      this.grid.scrollToCell({
        columnIndex: 0,
        rowIndex: index
      });
    }
  };

  _proto.shouldComponentUpdate = function shouldComponentUpdate(_ref11, nextState) {
    var update = _ref11.update,
        nextOther = _objectWithoutProperties(_ref11, ["update"]);

    var _props3 = this.props,
        _ = _props3.update,
        other = _objectWithoutProperties(_props3, ["update"]);

    return !shallowEqual(other, nextOther) || !shallowEqual(this.state, nextState) || update !== Update.None;
  };
  /**
   * Overrides openness state of nodes width ids specified as map keys to map values respectively.
   * @param map
   */


  _proto.toggleNodes =
  /*#__PURE__*/
  function () {
    var _toggleNodes = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee5(map) {
      var _id2;

      return _regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              // tslint:disable-next-line:forin no-for-in
              for (_id2 in map) {
                this.registry[_id2].isOpened = map[_id2];
              }

              _context5.next = 3;
              return this.recomputeTree(Update.Nodes);

            case 3:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5, this);
    }));

    return function toggleNodes(_x3) {
      return _toggleNodes.apply(this, arguments);
    };
  }();

  return Tree;
}(React.Component);

Tree.defaultProps = {
  autoHeight: false,
  estimatedRowSize: 30,
  noRowsRenderer: function noRowsRenderer() {
    return null;
  },
  nodeNestingMultiplier: 10,
  onRowsRendered: function onRowsRendered() {
    return undefined;
  },
  onScroll: function onScroll() {
    return undefined;
  },
  overscanIndicesGetter: accessibilityOverscanIndicesGetter,
  overscanRowCount: 10,
  rowRenderer: defaultRowRenderer,
  scrollToAlignment: 'auto',
  scrollToIndex: -1,
  style: {},
  update: Update.None
};
export { Tree as default };